from __future__ import annotations

from typing import TYPE_CHECKING

from dandy.llm import BaseLlmBot, LlmConfigOptions
from dandy.intel import BaseIntel
from dandy.llm import Prompt

if TYPE_CHECKING:
    from django_spire.seeding.helper import SeedHelper


class SeedLlmBot(BaseLlmBot):
    config = 'SEEDING_LLM_BOT'
    config_options = LlmConfigOptions(
        temperature=0.5
    )
    
    instructions_prompt = (
        Prompt()
        .title('You are a database seeding bot.')
        .text('Below you will find rules and instructions.')
        .text('Rules are specific per field and must be followed.')
        .text('Instructions have context behind the meaning of the data and how it should be created.')
    )

    @classmethod
    def process(
            cls,
            seed_helper: SeedHelper = None
    ):
        intel_class = seed_helper.build_intel_class()

        class SeedingIntel(BaseIntel):
            seeds: list[intel_class]

            def __iter__(self):
                return iter(self.seeds)

        return cls.process_prompt_to_intel(
            prompt=(
                Prompt()
                .title('Instructions')
                .prompt(seed_helper.seeding_prompt)
                .line_break()
                .text(f'Create {seed_helper.count} {seed_helper.model_class.__name__} objects.')
            ),
            intel_class=SeedingIntel
        )



from __future__ import annotations

from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.timezone import localtime

from django_spire.history.querysets import ActivityQuerySet


EVENT_HISTORY_CHOICES = (
    ('crea', 'Created'),
    ('upda', 'Updated'),
    ('acti', 'Active'),
    ('inac', 'Inactive'),
    ('dele', 'Deleted'),
    ('unde', 'Un-Deleted'),
)


class ActivityLog(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, editable=False)
    object_id = models.PositiveIntegerField(editable=False)
    content_object = GenericForeignKey('content_type', 'object_id')

    user = models.ForeignKey(
        User,
        related_name='users',
        related_query_name='user',
        on_delete=models.CASCADE,
        editable=False
    )

    recipient = models.ForeignKey(
        User,
        related_name='actors',
        related_query_name='actor',
        on_delete=models.CASCADE,
        editable=False,
        blank=True,
        null=True
    )

    verb = models.CharField(max_length=64)
    information = models.TextField(null=True, blank=True)

    created_datetime = models.DateTimeField(default=localtime)

    objects = ActivityQuerySet.as_manager()

    def __str__(self):
        return f'{self.user} - {self.verb}'

    def add_subscriber(self, subscriber: User) -> None:
        self.subscribers.create(
            user=subscriber
        )

    class Meta:
        verbose_name = 'Activities'
        verbose_name_plural = 'Activities'
        ordering = ['-created_datetime']


class ActivitySubscriber(models.Model):
    activity = models.ForeignKey(
        'spire_history.ActivityLog',
        on_delete=models.CASCADE,
        related_name='subscribers',
        related_query_name='subscriber'
    )

    subscriber = models.ForeignKey(
        User,
        related_name='activity_subscribers',
        related_query_name='activity_subscriber',
        on_delete=models.CASCADE,
        editable=False
    )

    created_datetime = models.DateTimeField(default=localtime)

    def __str__(self):
        return f'{self.activity} - {self.subscriber}'


class EventHistory(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, editable=False)
    object_id = models.PositiveIntegerField(editable=False)
    content_object = GenericForeignKey('content_type', 'object_id')

    event = models.CharField(max_length=4, choices=EVENT_HISTORY_CHOICES, default='upda')

    created_datetime = models.DateTimeField(default=localtime)

    def __str__(self) -> str:
        return f'{self.content_object} - {self.event_verbose}'

    class Meta:
        verbose_name = 'Event History'
        verbose_name_plural = 'Event History'

    @property
    def event_verbose(self) -> str:
        return dict(EVENT_HISTORY_CHOICES)[self.event]


class View(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, editable=False)
    object_id = models.PositiveIntegerField(editable=False)
    content_object = GenericForeignKey('content_type', 'object_id')

    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        editable=False,
        related_name='views',
        related_query_name='view'
    )

    created_datetime = models.DateTimeField(default=localtime, editable=False)

    def __str__(self):
        return f'{self.user} viewed {self.content_object} at {self.created_datetime}'

    class Meta:
        verbose_name = 'View'
        verbose_name_plural = 'Views'



from __future__ import annotations

from enum import Enum

from django.db import models

from django_spire.notification.sender import EmailNotificationSender


class NotificationTypeChoices(models.TextChoices):
    EMAIL = 'email'
    # SMS = 'sms'
    # PUSH = 'push'

    def send(self) -> None:
        sender_class = NotificationSenderEnum(self.value).value
        sender = sender_class(self)
        sender.send()


class NotificationSenderEnum(Enum):
    EMAIL = EmailNotificationSender
    # SMS = EmailNotificationSender
    # PUSH = EmailNotificationSender



from __future__ import annotations

from django.db.models import QuerySet
from django.utils.timezone import localtime


class MfaCodeQuerySet(QuerySet):
    def valid_code(self, user):
        return self.filter(user=user, expiration_datetime__gte=localtime()).first()



from django.db import models


class PersonSeedingModel(models.Model):
    first_name = models.CharField(max_length=16)
    last_name = models.CharField(max_length=32)
    phone_number = models.CharField(max_length=16)
    age = models.IntegerField()
    birth_date = models.DateField()

    class Meta:
        db_table = 'spire_person_seeding'
        verbose_name = 'Person Seeding'
        verbose_name_plural = 'Person Seeding'
        managed = False



from __future__ import annotations

import random
import string

from django import template
from django.urls import reverse

register = template.Library()


@register.filter
def add_str(str1: str, str2: str) -> str:
    return f'{str1}{str2}'


@register.simple_tag()
def content_type_url(url_name, obj, **kwargs):
    kwargs['app_label'] = obj._meta.app_label
    kwargs['model_name'] = obj._meta.model_name
    return reverse(url_name, kwargs=kwargs)


@register.filter
def in_list(value: str, arg: str):
    return value in arg.split(',')


@register.filter
def index(indexable, index_value):
    try:
        return indexable[index_value]
    except IndexError:
        return indexable


@register.simple_tag()
def generate_id():
    random_string = ''
    for i in range(0, 8):
        random_string = random_string + random.choice(string.ascii_letters)
    return random_string


@register.filter
def not_in_list(value, arg):
    return value not in arg.split(',')


@register.simple_tag()
def query_param_url(context, url_name, **kwargs):
    query_string = '?'

    for index, query_param in enumerate(context.request.GET):
        if index == 0:
            query_string = query_string + f'{query_param}={context.request.GET[query_param]}'
        else:
            query_string = query_string + f'&{query_param}={context.request.GET[query_param]}'

    url = reverse(url_name, kwargs=kwargs)

    return url + query_string


@register.simple_tag()
def to_snake_case(label: str) -> str:
    return label.replace(' ', '_').lower()



from __future__ import annotations

from django.db import models
from django_spire.options.options import Options

DEFAULT_OPTIONS = Options(sections=[])


class OptionsModelMixin(models.Model):
    _options = models.JSONField(default=dict)
    _default_options = DEFAULT_OPTIONS

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.options = Options.load_dict(self._options)

    def _sync_options(self):
        if self.options != self._default_options:
            self.options.sync_options(self._default_options)
            self._options = self.options.to_dict()

    def get_option(self, section_name: str, option_key: str):
        self._sync_options()
        return self.options.get_setting(section_name, option_key)

    def update_session(self, request):
        request.session['user_options'] = self.options.to_dict()

    def update_option(
            self,
            section_name: str,
            option_key: str,
            value: str | bool | int,
            commit=True
    ):
        self._sync_options()
        self.options.update_setting(section_name, option_key, value)
        self._options = self.options.to_dict()

        if commit:
            self.save()

    class Meta:
        abstract = True



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.conf import settings
from django.http import HttpResponseRedirect
from django.urls import reverse

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


class MaintenanceModeMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request: WSGIRequest):
        if settings.MAINTENANCE_MODE and 'maintenance/mode' not in request.path:
            url = reverse('home:maintenance_mode') + f'?next={request.path}'
            return HttpResponseRedirect(url)

        return self.get_response(request)



from __future__ import annotations

from typing_extensions import TYPE_CHECKING
from urllib.parse import parse_qs, unquote, urlencode, urlparse, urlunparse

from django.urls import reverse, NoReverseMatch
from django.utils.http import url_has_allowed_host_and_scheme
from django.conf import settings

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def resolve_url(url: str) -> str:
    try:
        return reverse(url)
    except NoReverseMatch:
        return url


def is_url_valid_and_safe(url: str, allowed_hosts: set[str]) -> bool:
    if url and url_has_allowed_host_and_scheme(url, allowed_hosts):
        url = urlparse(url)

        harmful = ['<', '>', '"', '{', '}', '|', '\\', '^', '`', '[', ']', ';']

        if any(character in unquote(url.path) for character in harmful):
            return False

        return not any(
            character in unquote(url.query)
            for character in harmful
        )

    return False


def safe_redirect_url(request: WSGIRequest, fallback: str = '/') -> str:
    allowed_hosts = {request.get_host()}

    if hasattr(settings, 'ALLOWED_HOSTS'):
        allowed_hosts.update(settings.ALLOWED_HOSTS)

    return_url = request.GET.get('return_url')

    if is_url_valid_and_safe(url=return_url, allowed_hosts=allowed_hosts):
        return resolve_url(return_url)

    referer = request.META.get('HTTP_REFERER')
    print(referer)

    if is_url_valid_and_safe(url=referer, allowed_hosts=allowed_hosts):
        url = urlparse(referer)
        query_string = urlencode(parse_qs(url.query), doseq=True)
        path = resolve_url(url.path)

        full = (
            url.scheme,
            url.netloc,
            path,
            '',
            query_string,
            url.fragment
        )

        return urlunparse(full)

    if not is_url_valid_and_safe(url=fallback, allowed_hosts=allowed_hosts):
        fallback = '/'

    return fallback



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.conf import settings
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.breadcrumb.models import Breadcrumbs
from django_spire.views import portal_views

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def home_view(request: WSGIRequest) -> TemplateResponse:
    crumbs = Breadcrumbs()
    crumbs.add_breadcrumb(name='Home', href=reverse('home:home'))

    return portal_views.template_view(
        request,
        page_title='Home',
        page_description='Your Portal',
        breadcrumbs=crumbs,
        template='spire/home/page/home_page.html'
    )


def maintenance_mode_view(request: WSGIRequest) -> TemplateResponse | HttpResponseRedirect:
    if not settings.MAINTENANCE_MODE:
        return HttpResponseRedirect(
            request.GET.get(
                'next',
                reverse('home:home')
            )
        )

    return TemplateResponse(
        request,
        template='spire/home/page/maintenance_mode_page.html'
    )



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django_spire.consts import VERSION

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def spire(_: WSGIRequest) -> dict[str, str]:
    return {'DJANGO_SPIRE_VERSION': VERSION}



from __future__ import annotations

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Row, Column, Submit

from django import forms
from django_spire.file.fields import MultipleFileField


class FileForm(forms.Form):
    files = MultipleFileField()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.helper = FormHelper(self)
        self.helper.include_media = False
        self.helper.form_enctype = 'multipart/form-data'
        self.helper.layout = Layout(
            Row(
                Column('files', css_class='form-group col-md-6'),
            ),
        )
        self.helper.add_input(Submit('submit', 'Submit', css_class='btn-primary'))




from __future__ import annotations

import json

from django.contrib.contenttypes.models import ContentType

from django.contrib.contenttypes.fields import GenericForeignKey
from django.db import models

from django_spire.file.queryset import FileQuerySet
from django_spire.history.mixins import HistoryModelMixin


class File(HistoryModelMixin):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, editable=False, blank=True, null=True)
    object_id = models.PositiveIntegerField(editable=False, blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    file = models.FileField(max_length=500)
    name = models.CharField(max_length=255, default='', editable=False)
    type = models.CharField(max_length=255, default='', editable=False)
    size = models.CharField(max_length=64, default='', editable=False)

    related_field = models.CharField(max_length=3, null=True, blank=True, editable=False)

    objects = FileQuerySet.as_manager()

    def __str__(self):
        return self.name

    def to_dict(self) -> dict[str, str]:
        return {
            'name': self.name,
            'url': self.file.url,
            'id': self.id
        }

    def to_json(self) -> str:
        return json.dumps(self.to_dict())

    class Meta:
        db_table = 'spire_file'
        verbose_name = 'File'
        verbose_name_plural = 'Files'
        indexes = [
            models.Index(fields=['content_type', 'object_id']),
        ]




from __future__ import annotations

from django.test import Client, TestCase

from django_spire.user_account.tests.factories import create_super_user


class BaseTestCase(TestCase):
    def setUp(self):
        self.client = Client()

        self.super_user = create_super_user()
        self.client.force_login(self.super_user)

    def tearDown(self):
        pass



from django.db.models import Field

from pydantic.fields import Field


class SeedIntelFieldFactory:

    def __init__(self, model_field: Field):
        self.model_field = model_field

    def build_field(self) -> Field:
        kwargs = {
            'description': '',
        }

        if self.model_field.null:
            kwargs['default'] = None

        if self.model_field.get_internal_type() in ['DateField']:
            kwargs['description'] += 'Date Format: YYYY-MM-DD '
            kwargs['examples'] = ['2022-01-01']

        if self.model_field.get_internal_type() in ['DateTimeField']:
            kwargs['description'] += 'Date Format: YYYY-MM-DD HH:MM:SS '
            kwargs['examples'] = ['2022-01-01 13:37:00']

        if self.model_field.unique:
            kwargs['description'] += 'Is Unique: True '

        if self.model_field.max_length:
            kwargs['max_length'] = int(self.model_field.max_length)

        return Field(**kwargs)


# Generated by Django 5.1.3 on 2024-11-08 22:57

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, editable=False)),
                ('is_deleted', models.BooleanField(default=False, editable=False)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime, editable=False)),
                ('object_id', models.PositiveIntegerField(blank=True, editable=False, null=True)),
                ('file', models.FileField(max_length=500, upload_to='')),
                ('name', models.CharField(default='', editable=False, max_length=255)),
                ('type', models.CharField(default='', editable=False, max_length=255)),
                ('size', models.CharField(default='', editable=False, max_length=64)),
                ('related_field', models.CharField(blank=True, editable=False, max_length=3, null=True)),
                ('content_type', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name': 'File',
                'verbose_name_plural': 'Files',
                'db_table': 'spire_file',
                'indexes': [models.Index(fields=['content_type', 'object_id'], name='spire_file_content_04cd73_idx')],
            },
        ),
    ]



from __future__ import annotations

from django.db.models import QuerySet


class ActivityQuerySet(QuerySet):
    def prefetch_user(self):
        return self.prefetch_related('user')


class HistoryQuerySet(QuerySet):
    def active(self):
        return self.filter(is_active=True, is_deleted=False)

    def deleted(self):
        return self.filter(is_deleted=True)



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse

from django_spire.form.utils import show_form_errors
from django_spire.views import portal_views
from django_spire.core.shortcuts import get_object_or_null_obj
from django_spire.history.utils import add_form_activity

from module import forms, models

from django_glue.glue import glue_model

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.template.response import TemplateResponse


def placeholder_delete_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    placeholder = get_object_or_404(models.Placeholder, pk=pk)

    return_url = request.GET.get(
        'return_url',
        reverse('placeholder:page:list')
    )

    return portal_views.delete_form_view(
        request,
        obj=placeholder,
        return_url=return_url
    )


def placeholder_detail_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    placeholder = get_object_or_404(models.Placeholder, pk=pk)

    context_data = {
        'placeholder': placeholder,
    }

    return portal_views.detail_view(
        request,
        obj=placeholder,
        context_data=context_data,
        template='placeholder/page/placeholder_detail_page.html'
    )


def placeholder_form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    placeholder = get_object_or_null_obj(models.Placeholder, pk=pk)

    glue_model(request, 'placeholder', placeholder, 'view')

    if request.method == 'POST':
        form = forms.PlaceholderForm(request.POST, instance=placeholder)

        if form.is_valid():
            placeholder = form.save()
            add_form_activity(placeholder, pk, request.user)

            return redirect(
                request.GET.get(
                    'return_url',
                    reverse('placeholder:page:list')
                )
            )

        show_form_errors(request, form)
    else:
        form = forms.PlaceholderForm(instance=placeholder)

    return portal_views.form_view(
        request,
        form=form,
        obj=placeholder,
        template='placeholder/page/placeholder_form_page.html'
    )


def placeholder_list_view(request: WSGIRequest) -> TemplateResponse:
    context_data = {
        'placeholders': models.Placeholder.objects.all()
    }

    return portal_views.list_view(
        request,
        model=models.Placeholder,
        context_data=context_data,
        template='placeholder/page/placeholder_list_page.html'
    )



from dataclasses import dataclass, field
from typing import Type

from dandy.intel import BaseIntel
from dandy.llm import Prompt
from django.db.models import Model
from pydantic import create_model

from django_spire.seeding.maps import MODEL_FIELD_TYPE_TO_TYPE_MAP
from django_spire.seeding.factories import SeedIntelFieldFactory
from django_spire.seeding.intelligence.bots.seeding_bot import SeedLlmBot


@dataclass
class SeedHelper:
    model_class: Type[Model]

    seeding_prompt: Prompt
    
    count: int = 5

    exclude_fields: list[str] = field(default_factory=list)
    include_fields: list[str] = field(default_factory=list)

    @property
    def valid_model_fields(self):
        return [
            f
            for f in self.model_class._meta.fields
            if (not self.include_fields or f.attname in self.include_fields)
               and
               (f.attname not in self.exclude_fields)
        ]

    def build_intel_class(self) -> Type[BaseIntel]:
        pydantic_fields = {}

        for model_field in self.valid_model_fields:

            model_field_type = MODEL_FIELD_TYPE_TO_TYPE_MAP[model_field.get_internal_type()]

            if model_field.null:
                model_field_type = model_field_type | None

            pydantic_fields[model_field.attname] = (
                model_field_type,
                SeedIntelFieldFactory(model_field).build_field()
            )

        intel_class = create_model(
            f'{self.model_class.__name__}Intel',
            __base__=BaseIntel,
            **pydantic_fields
        )

        return intel_class

    def bulk_create(self, intel_data: list[BaseIntel]):
        instances = [
            self.model_class(**intel.model_dump())
            for intel in intel_data
        ]

        return self.model_class.objects.bulk_create(instances)

    def seed(self):
        self.bulk_create(
            SeedLlmBot.process(self)
        )



from __future__ import annotations

from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.timezone import now

from django_spire.comment.querysets import CommentQuerySet
from django_spire.history.mixins import HistoryModelMixin


class Comment(HistoryModelMixin):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, editable=False)
    object_id = models.PositiveIntegerField(editable=False)
    content_object = GenericForeignKey('content_type', 'object_id')

    parent = models.ForeignKey(
        'self',
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        related_name='children',
        related_query_name='child'
    )

    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        editable=False,
        related_name='comment_list'
    )

    information = models.TextField(default='')
    created_datetime = models.DateTimeField(default=now, editable=False)
    is_edited = models.BooleanField(default=False, editable=False)

    objects = CommentQuerySet.as_manager()

    def __str__(self):
        return f'{self.information[0:10]}...'

    # def active_replies(self):
    #     return self.children.active()

    def find_user_list(self):
        from django.contrib.auth.models import User
        return User.objects.filter(username__in=self.scrape_username_list())

    def scrape_username_list(self):
        return [username[1:] for username in self.information.split() if username.startswith('@')]

    def send_notification(self):
        print('Function Called')
        print(self.scrape_username_list())
        print(self.find_user_list())
        for user in self.find_user_list():
            print(f'Sending comment to {user.username}')
            # create(
            #     user=user,
            #     head=f'{self.user.get_full_name()} commented on a task',
            #     subject='Business Base - New Comment',
            #     body=f'"{self.information}" on {self.content_object}',
            #     # Todo: Need to generate URL. Use generic reverse?
            #     url='',
            # )

    # Ordering by statistic_date_time_created creates duplicated results when querying because of generic relationship.
    def update(self, information):
        self.information = information
        self.is_edited = True
        self.save()

    class Meta:
        db_table = 'spire_comment'
        verbose_name = 'Comment'
        verbose_name_plural = 'Comments'
        ordering = ['-created_datetime']



from __future__ import annotations

from django import template


register = template.Library()


@register.simple_tag(takes_context=True)
def pagination_url(
    context,
    page_number: int,
    page_name: str = 'page',
    **kwargs
):
    updated_context = context.request.GET.copy()
    updated_context[page_name] = page_number
    query_string = '?'

    for index, query_param in enumerate(updated_context):
        query_param_value = updated_context[query_param]
        if isinstance(query_param_value, str):
            # Replacing space with + sign to match how query parameters and django form filtering.
            query_param_value = query_param_value.replace(' ', '+')

        if index == 0:
            query_string = query_string + f'{query_param}={query_param_value}'
        else:
            query_string = query_string + f'&{query_param}={query_param_value}'

    return query_string


@register.simple_tag
def get_elided_page_range(
    page_obj,
    on_each_side: int = 2,
    on_ends: int = 2
):
    return page_obj.paginator.get_elided_page_range(
        number=page_obj.number,
        on_each_side=on_each_side,
        on_ends=on_ends
    )



from __future__ import annotations

from django_spire.permission.models import PortalUser


def register_new_user(
    first_name: str,
    last_name: str,
    email: str,
    password: str
) -> PortalUser:
    return PortalUser.objects.create_user(
        username=email.lower(),
        email=email.lower(),
        password=password,
        first_name=first_name,
        last_name=last_name
    )



from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html

from django_spire.authentication.mfa import models


@admin.register(models.MfaCode)
class MfaCodeAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'user_link', 'code', 'expiration_datetime', 'is_valid'
    )
    list_filter = ('expiration_datetime',)
    search_fields = ('id', 'user__username', 'code')
    ordering = ('-expiration_datetime',)

    def user_link(self, mfa_code: models.MfaCode) -> str:
        url = reverse('admin:auth_user_change', args=[mfa_code.user.id])
        return format_html(f'<a href="{url}">{mfa_code.user.username}</a>')

    user_link.short_description = 'User'



from dandy.llm import Prompt
from django.test import TestCase

from django_spire.seeding.helper import SeedHelper
from django_spire.seeding.models import PersonSeedingModel


class SeedingTestCase(TestCase):
    def setUp(self):
        self.person_seed_helper = SeedHelper(
            model_class=PersonSeedingModel,
            seeding_prompt=(
                Prompt()
                .text('Create a bunch of unique people to be used for seeding.')
            ),
            count=5,
            exclude_fields=['phone_number'],
        )

    def tearDown(self):
        pass

    def test_seeding_helper_intel_class(self):
        person_intel_class = self.person_seed_helper.build_intel_class()
        
        print(person_intel_class.model_json_schema())
